@@ -90,9 +90,7 @@ Encrypt. To do the same but skip connectivity checks, use
 C<--acme-always> flag instead.
 
 The C<--proxy> parameter can be used to have the website proxy all requests
-to another URL, which must follow C<--proxy>. Alternately, the C<--framefwd>
-parameter similarly can be used to forward requests to the virtual server to
-another URL, using a hidden frame rather than proxying.
+to another URL, which must follow C<--proxy>.
 
 =cut
 
@@ -466,10 +464,6 @@ while(@ARGV > 0) {
 		$proxy_pass_mode = 1;
 		$proxy_pass = shift(@ARGV);
 		}
-	elsif ($a eq "--framefwd") {
-		$proxy_pass_mode = 2;
-		$proxy_pass = shift(@ARGV);
-		}
 	elsif ($a eq "--default-cert-owner") {
 		$default_cert_owner = 1;
 		}
@@ -953,6 +947,7 @@ if (!$parent) {
 	}
 $dom{'emailto'} = $parent ? $parent->{'emailto'} :
 		  $dom{'email'} ? $dom{'email'} :
+		  $dom{'user'} =~ /\@/ ? $dom{'user'} :
 		  $dom{'mail'} ? $dom{'user'}.'@'.$dom{'dom'} :
 		  		 $dom{'user'}.'@'.&get_system_hostname();
 foreach $f (@features) {
@@ -1197,7 +1192,7 @@ print "                        [--generate-ssh-key | --use-ssh-key file|data]\n"
 print "                        [--append-style format]\n";
 print "                        [--shell command]\n";
 print "                        [--subprefix directory]\n";
-print "                        [--proxy url | --framefwd url]\n";
+print "                        [--proxy url]\n";
 exit(1);
 }
 
