@@ -411,7 +411,6 @@ elsif (@{$dkim->{'extra'}}) {
 	}
 else {
 	&$second_print($text{'dkim_nodomains'});
-	return 0;
 	}
 
 # Generate private key
@@ -887,7 +886,8 @@ return 1;
 }
 
 # can_dkim_domain(&domain, &dkim)
-# Returns 1 if a domain should have DKIM enabled by default
+# Returns 1 if a domain should have DKIM enabled by default, 0 if it should
+# not be, or 2 if it is enabled for all domains
 sub can_dkim_domain
 {
 my ($d, $dkim) = @_;
@@ -912,13 +912,16 @@ else {
 	}
 }
 
-# has_dkim_domain(&domain, &dkim)
-# Returns 1 if a domain must have DKIM enabled
+# has_dkim_domain(&domain, [&dkim])
+# Returns 1 if a domain should have DKIM enabled, based on current features
+# and any domain-level override.
 sub has_dkim_domain
 {
 my ($d, $dkim) = @_;
+$dkim ||= { 'alldns' => $config{'dkim_alldns'} || 0 };
 my $can = &can_dkim_domain($d, $dkim);
 if ($can != 2) {
+	# DKIM is not enabled for all domains, so it depends on DNS
 	return 0 if (!$d->{'dns'});
 	}
 return 0 if (defined($d->{'dkim_enabled'}) && $d->{'dkim_enabled'} eq '0');
@@ -926,36 +929,52 @@ return 1 if (defined($d->{'dkim_enabled'}) && $d->{'dkim_enabled'} eq '1');
 return $can;
 }
 
-# update_dkim_domains(&domain, action, [no-dns])
-# Updates the list of domains to sign mail for, if needed
-sub update_dkim_domains
+# sync_dkim_domain(&domain)
+# Enable or disable DKIM for a domain based on it's current enabled features
+# and the global DKIM setting
+sub sync_dkim_domain
 {
-my ($d, $action, $nodns) = @_;
+my ($d) = @_;
+
+# Get global DKIM config
 return if (&check_dkim());
-&lock_file(&get_dkim_config_file());
+my $cfile = &get_dkim_config_file();
+&lock_file($cfile);
 my $dkim = &get_dkim_config();
-return if (!$dkim || !$dkim->{'enabled'});
+if (!$dkim || !$dkim->{'enabled'}) {
+	&unlock_file($cfile);
+	return;
+	}
 
-# Enable DKIM for all domains with mail
+# First update the complete list of domains to enable DKIM for, in the
+# OpenDKIM config file
+my @olddoms = &get_dkim_domains($dkim);
 my @doms = grep { &has_dkim_domain($_, $dkim) } &list_domains();
-if (($action eq 'setup' || $action eq 'modify')) {
-	push(@doms, $d);
+my ($olddom) = grep { $_ eq $d->{'dom'} } @olddoms;
+my ($newdom) = grep { $_->{'id'} eq $d->{'id'} } @doms;
+my $msg;
+if ($newdom && !$olddom) {
+	$msg = &text('dkim_adding', &show_domain_name($d));
 	}
-elsif ($action eq 'delete') {
-	@doms = grep { $_->{'id'} ne $d->{'id'} } @doms;
+elsif (!$newdom && $olddom) {
+	$msg = &text('dkim_removing', &show_domain_name($d));
 	}
-my %done;
-@doms = grep { !$done{$_->{'id'}}++ } @doms;
+&$first_print($msg) if ($msg);
 &set_dkim_domains(\@doms, $dkim);
-&unlock_file(&get_dkim_config_file());
+&$second_print($text{'setup_done'}) if ($msg);
+&unlock_file($cfile);
 
-# Add or remove DNS records
-if ($d->{'dns'} && !&copy_alias_records($d) && !$nodns) {
-	if ($action eq 'setup' || $action eq 'modify') {
-		&add_dkim_dns_records([ $d ], $dkim);
-		}
-	elsif ($action eq 'delete') {
-		&remove_dkim_dns_records([ $d ], $dkim);
+if (!&copy_alias_records($d) && $d->{'dns'}) {
+	my ($err, $file) = &get_domain_dns_records_and_file($d);
+	if ($file) {
+		if (&has_dkim_domain($d, $dkim)) {
+			# Make sure DNS records exist
+			&add_dkim_dns_records([ $d ], $dkim);
+			}
+		else {
+			# Remove any DNS records
+			&remove_dkim_dns_records([ $d ], $dkim);
+			}
 		}
 	}
 }
@@ -1074,14 +1093,22 @@ sub add_dkim_dns_records
 my ($doms, $dkim) = @_;
 my $anychanged = 0;
 foreach my $d (@$doms) {
+	# First check if records are already OK
+	my ($recs, $file) = &get_domain_dns_records_and_file($d);
+	next if (!$file);	# Should never happen
+	my ($dkrec, $selrecs) = &has_domain_dkim_record($d, $dkim, $recs);
+	if (@$selrecs) {
+		my $v = join("", @{$selrecs->[0]->{'values'}});
+		my $pubkey = &get_dkim_dns_pubkey($dkim, $d);
+		if ($v =~ /p=\Q$pubkey\E/) {
+			# Record already exists, and looks OK. So we don't need
+			# to update or add records for ths domain
+			next;
+			}
+		}
+
 	&$first_print(&text('dkim_dns', "<tt>$d->{'dom'}</tt>"));
 	&pre_records_change($d);
-	my ($recs, $file) = &get_domain_dns_records_and_file($d);
-	if (!$file) {
-		&after_records_change($d);
-		&$second_print($text{'dkim_ednszone'});
-		next;
-		}
 	&obtain_lock_dns($d);
 	my $changed = &add_domain_dkim_record($d, $dkim, $recs, $file);
 	if ($changed) {
@@ -1100,6 +1127,7 @@ foreach my $d (@$doms) {
 		}
 	&release_lock_dns($d);
 	}
+&register_post_action(\&restart_bind, $d) if ($anychanged);
 }
 
 # add_domain_dkim_record(&domain, &dkim, &recs, file)
@@ -1107,23 +1135,22 @@ foreach my $d (@$doms) {
 sub add_domain_dkim_record
 {
 my ($d, $dkim, $recs, $file) = @_;
+my ($dkrec, $selrecs) = &has_domain_dkim_record($d, $dkim, $recs);
+my $selrec = @$seclrecs ? $selrecs->[0] : undef;
 my $withdot = $d->{'dom'}.'.';
-my $dkname = '_domainkey.'.$withdot;
-my $changed = 0;
-my $selname = $dkim->{'selector'}.'.'.$dkname;
-my ($selrec) = grep { $_->{'name'} eq $selname && 
-		      $_->{'type'} eq 'TXT' } @$recs;
 my $pubkey = &get_dkim_dns_pubkey($dkim, $d);
+
+my $changed = 0;
 if (!$selrec) {
 	# Add new record
-	my $selrec = { 'name' => $selname,
+	my $selrec = { 'name' => $dkim->{'selector'}.'.'.'_domainkey.'.$withdot,
 		       'type' => 'TXT',
 		       'values' => [ 'v=DKIM1; k=rsa; t=s; p='.
 				     $pubkey ] };
 	&create_dns_record($recs, $file, $selrec);
 	$changed++;
 	}
-elsif ($selrec && join("", @{$selrec->{'values'}}) !~ /p=\Q$pubkey\E/) {
+elsif (join("", @{$selrec->{'values'}}) !~ /p=\Q$pubkey\E/) {
 	# Fix existing record
 	my $val = join("", @{$selrec->{'values'}});
 	if ($val !~ s/p=([^;]+)/p=$pubkey/) {
@@ -1136,23 +1163,21 @@ elsif ($selrec && join("", @{$selrec->{'values'}}) !~ /p=\Q$pubkey\E/) {
 return $changed;
 }
 
-# remove_dkim_dns_records(&domains, &dkim)
+# remove_dkim_dns_records(&domains, &dkim, [all-selectors])
 # Delete all DKIM TXT records from the given DNS domains
 sub remove_dkim_dns_records
 {
-my ($doms, $dkim) = @_;
+my ($doms, $dkim, $all) = @_;
 my $anychanged = 0;
 foreach my $d (@$doms) {
+	my ($recs, $file) = &get_domain_dns_records_and_file($d);
+	next if (!$file);       # Should never happen
+        my ($dkrec, $selrecs) = &has_domain_dkim_record($d, $dkim, $recs);
+	next if (!$dkrec && !@$selrecs);
 	&$first_print(&text('dkim_undns', "<tt>$d->{'dom'}</tt>"));
 	&pre_records_change($d);
-	my ($recs, $file) = &get_domain_dns_records_and_file($d);
-	if (!$file) {
-		&after_records_change($d);
-		&$second_print($text{'dkim_ednszone'});
-		next;
-		}
 	&obtain_lock_dns($d);
-	my $changed = &remove_domain_dkim_record($d, $dkim, $recs, $file);
+	my $changed = &remove_domain_dkim_record($d, $dkim, $recs, $file, $all);
 	if ($changed) {
 		&post_records_change($d, $recs, $file);
 		&$second_print($text{'dkim_dnsremoved'});
@@ -1164,22 +1189,17 @@ foreach my $d (@$doms) {
 		}
 	&release_lock_dns($d);
 	}
+&register_post_action(\&restart_bind, $d) if ($anychanged);
 }
 
-# remove_domain_dkim_record(&domain, &dkim, &recs, file)
+# remove_domain_dkim_record(&domain, &dkim, &recs, file, [all-selectors])
 # Remove the DKIM records for a single domain from its zone file
 sub remove_domain_dkim_record
 {
-my ($d, $dkim, $recs, $file) = @_;
-my $withdot = $d->{'dom'}.'.';
-my $dkname = '_domainkey.'.$withdot;
-my ($dkrec) = grep { $_->{'name'} eq $dkname &&
-		     $_->{'type'} eq 'TXT' } @$recs;
-my $selname = $dkim->{'selector'}.'.'.$dkname;
-my ($selrec) = grep { $_->{'name'} eq $selname &&
-		      $_->{'type'} eq 'TXT' } @$recs;
+my ($d, $dkim, $recs, $file, $all) = @_;
+my ($dkrec, $selrecs) = &has_domain_dkim_record($d, $dkim, $recs, $all);
 my $changed = 0;
-if ($selrec) {
+foreach my $selrec (@$selrecs) {
 	&delete_dns_record($recs, $selrec->{'file'}, $selrec);
 	$changed++;
 	}
@@ -1190,6 +1210,28 @@ if ($dkrec) {
 return $changed;
 }
 
+# has_domain_dkim_record(&domain, &dkim, &recs, [all-selectors])
+# Returns the DKIM primary record and selector records, where they exist
+sub has_domain_dkim_record
+{
+my ($d, $dkim, $recs, $all) = @_;
+my $withdot = $d->{'dom'}.'.';
+my $dkname = '_domainkey.'.$withdot;
+my ($dkrec) = grep { $_->{'name'} eq $dkname &&
+                     $_->{'type'} eq 'TXT' } @$recs;
+my @selrecs;
+if ($all) {
+        @selrecs = grep { $_->{'name'} =~ /^([^\.]+)\.\Q$dkname\E$/ &&
+                          $_->{'type'} eq 'TXT' } @$recs;
+        }
+else {
+        my $selname = $dkim->{'selector'}.'.'.$dkname;
+        @selrecs = grep { $_->{'name'} eq $selname &&
+                          $_->{'type'} eq 'TXT' } @$recs;
+        }
+return ($dkrec, \@selrecs);
+}
+
 # rebuild_sendmail_cf()
 # Rebuild sendmail's .cf file from the .mc file
 sub rebuild_sendmail_cf
@@ -1475,5 +1517,18 @@ my @tm = localtime(time());
 return sprintf "%4.4d%2.2d", $tm[5] + 1900, $tm[4];
 }
 
+# refresh_dkim_dns(&domain)
+# If DKIM is enabled globally, add or update the DNS records for this domain
+sub refresh_dkim_dns
+{
+my ($d) = @_;
+my $dkim = &get_dkim_config();
+if ($dkim && $dkim->{'enabled'} && &has_dkim_domain($d, $dkim) &&
+    $d->{'dns'} && !&copy_alias_records($d)) {
+	&remove_dkim_dns_records([ $d ], $dkim, 1);
+	&add_dkim_dns_records([ $d ], $dkim);
+	}
+}
+
 1;
 
