@@ -125,24 +125,35 @@ foreach my $p (@ports) {
 			next if ($i > 0 &&
 				 $rws[$i-1]->{'name'} eq 'RewriteCond');
 			my $rwr = $rws[$i];
-			next if ($rwr->{'words'}->[2] !~ /^\[R(=\d+)?\]$/);
+			next if ($rwr->{'words'}->[2] !~ /^\[.*R(=\d+)?.*\]$/);
 			my @dirs = ( $rwr );
 			my $rd = { 'alias' => 0, 'dirs' => \@dirs };
 			$rd->{$proto} = 1;
 			$rd->{'path'} = $rwr->{'words'}->[0];
 			$rd->{'dest'} = $rwr->{'words'}->[1];
-			if ($rd->{'path'} =~ /^(.*)\.\*\$$/ ||
-			    $rd->{'path'} =~ /^(.*)\(\.\*\)\$$/) {
+			if ($rd->{'dest'} !~ /\$1$/ &&
+			    ($rd->{'path'} =~ /^(.*)\.\*\$$/ ||
+			     $rd->{'path'} =~ /^(.*)\(\.\*\)\$$/)) {
+				# Path is like /something(.*) but the destination
+				# does not include $1, meaning that sub-paths are
+				# matched but are not used
 				$rd->{'path'} = $1;
 				$rd->{'regexp'} = 1;
 				}
 			elsif ($rd->{'path'} =~ /^\^(.*)\$$/) {
-			       $rd->{'path'} = $1;
+				# Path is like ^/something$, meaning that sub-paths
+				# are not matched
+				$rd->{'path'} = $1;
 				$rd->{'exact'} = 1;
 				}
-			if ($rwr->{'words'}->[2] =~ /^\[R=(\d+)\]$/) {
-				$rd->{'code'} = $1;
+			elsif ($rd->{'path'} =~ /^(.*)\(\.\*\)\$$/) {
+				# Path is like /something(.*) and the destination
+				# includes $1, meaning that sub-paths are matched
+				# and included in the redirect
+                                $rd->{'path'} = $1;
+				$rd->{'dest'} =~ s/\$1$//;
 				}
+			&parse_rewritecond_flags($rwr->{'words'}->[2], $rd);
 			$rd->{'id'} = $rwr->{'name'}.'_'.$rd->{'path'};
 			my ($already) = 
 				grep { $_->{'path'} eq $rd->{'path'} &&
@@ -178,7 +189,7 @@ foreach my $p (@ports) {
 			$j++;
 			}
 		next if (!$rwr || !$rwc && !$rwh);
-		next if ($rwr->{'words'}->[2] !~ /^\[R(=\d+)?\]$/);
+		next if ($rwr->{'words'}->[2] !~ /^\[.*R(=\d+)?.*\]$/);
 		my @dirs = ( $rwr );
 		push(@dirs, $rwc) if ($rwc);
 		push(@dirs, $rwh) if ($rwh);
@@ -217,19 +228,30 @@ foreach my $p (@ports) {
 			}
 		$rd->{'path'} = $rwr->{'words'}->[0];
 		$rd->{'dest'} = $rwr->{'words'}->[1];
-		if ($rd->{'path'} =~ /^(.*)\.\*\$$/ ||
-		    $rd->{'path'} =~ /^(.*)\(\.\*\)\$$/) {
+		if ($rd->{'dest'} !~ /\$1$/ &&
+		    ($rd->{'path'} =~ /^(.*)\.\*\$$/ ||
+		     $rd->{'path'} =~ /^(.*)\(\.\*\)\$$/)) {
+			# Path is like /something(.*) but the destination
+			# does not include $1, meaning that sub-paths are
+			# matched but are not used
 			$rd->{'path'} = $1;
 			$rd->{'regexp'} = 1;
 			}
 		elsif ($rd->{'path'} =~ /^\^(.*)\$$/) {
+			# Path is like ^/something$, meaning that sub-paths
+			# are not matched
 			$rd->{'path'} = $1;
 			$rd->{'exact'} = 1;
 			}
-		if ($rwr->{'words'}->[2] =~ /^\[R=(\d+)\]$/) {
-			$rd->{'code'} = $1;
+		elsif ($rd->{'path'} =~ /^(.*)\(\.\*\)\$$/) {
+			# Path is like /something(.*) and the destination
+			# includes $1, meaning that sub-paths are matched
+			# and included in the redirect
+			$rd->{'path'} = $1;
+			$rd->{'dest'} =~ s/\$1$//;
 			}
-		$rd->{'id'} = $rwc->{'name'}.'_'.$rd->{'path'};
+		&parse_rewritecond_flags($rwr->{'words'}->[2], $rd);
+		$rd->{'id'} = $rwr->{'name'}.'_'.$rd->{'path'};
 		$rd->{'id'} .= '_'.$rd->{'host'} if ($rd->{'host'});
 		my ($already) = grep { $_->{'path'} eq $rd->{'path'} &&
 				       $_->{'host'} eq $rd->{'host'} } @rv;
@@ -245,6 +267,24 @@ foreach my $p (@ports) {
 return @rv;
 }
 
+# parse_rewritecond_flags(flags, &redirect)
+# Turn a flag string like [R=302,L] into settings in a redirect
+sub parse_rewritecond_flags
+{
+my ($flags, $rd) = @_;
+if ($flags =~ /^\[(.*)\]$/) {
+	my @flags = split(/,/, $1);
+	foreach my $f (@flags) {
+		if ($f =~ /^R=(\d+)$/) {
+			$rd->{'code'} = $1;
+			}
+		elsif ($f eq "L") {
+			$rd->{'last'} = 1;
+			}
+		}
+	}
+}
+
 # create_redirect(&domain, &redirect)
 # Creates a new alias or redirect in some domain
 sub create_redirect
@@ -278,17 +318,33 @@ foreach my $p (@ports) {
 		my @rwes = &apache::find_directive("RewriteEngine", $vconf);
 		my @rwcs = &apache::find_directive("RewriteCond", $vconf);
 		my @rwrs = &apache::find_directive("RewriteRule", $vconf);
-		my $flag = $redirect->{'code'} ? "[R=".$redirect->{'code'}."]"
-					       : "[R]";
+		my @flags;
+		push(@flags, $redirect->{'code'} ? "R=".$redirect->{'code'}
+						 : "R");
+		push(@flags, "L") if ($redirect->{'last'});
 		if ($redirect->{'host'}) {
 			push(@rwcs, "%{HTTP_HOST} ".
 			     ($redirect->{'hostregexp'} ? "" : "=").
 			     $redirect->{'host'});
 			}
-		my $path = $redirect->{'path'};
-		$path .= "(\.\*)\$" if ($redirect->{'regexp'});
-		$path = "^".$path."\$" if ($redirect->{'exact'});
-		push(@rwrs, $path." ".$redirect->{'dest'}." ".$flag);
+		my $path;
+		my $dest = $redirect->{'dest'};
+		if ($redirect->{'exact'}) {
+			# Only match the exact path, not any sub-paths
+			$path = "^".$redirect->{'path'}."\$";
+			}
+		elsif ($redirect->{'regexp'}) {
+			# Match all sub-paths, but do not include them
+			# in the redirect
+			$path = $redirect->{'path'}."(\.\*)\$";
+			}
+		else {
+			# Match all sub-paths, and include them in the
+			# redirect
+			$path = $redirect->{'path'}."(\.\*)\$";
+			$dest .= "\$1";
+			}
+		push(@rwrs, $path." ".$dest." [".join(",", @flags)."]");
 		if (!@rwes) {
 			&apache::save_directive(
 				"RewriteEngine", ["on"], $vconf, $conf);
@@ -299,17 +355,24 @@ foreach my $p (@ports) {
 	else {
 		# Can just use Alias or Redirect
 		my $dir = $redirect->{'alias'} ? "Alias" : "Redirect";
-		$dir .= "Match" if ($redirect->{'regexp'} ||
-				    $redirect->{'exact'});
-		my @aliases = &apache::find_directive($dir, $vconf);
 		my $path;
 		if ($redirect->{'exact'}) {
+			# Only match the exact path, not any sub-paths
 			$path = "^".$redirect->{'path'}."\$";
+			$dir .= "Match";
+			}
+		elsif ($redirect->{'regexp'}) {
+			# Match all sub-paths, but do not include them
+			# in the redirect
+			$path = $redirect->{'path'}."(\.\*)\$";
+			$dir .= "Match";
 			}
 		else {
-			$path = $redirect->{'path'}.
-				($redirect->{'regexp'} ? "(\.\*)\$" : "");
+			# Just use a bare redirect which automatically matches
+			# sub-paths and includes them in the redirect
+			$path = $redirect->{'path'};
 			}
+		my @aliases = &apache::find_directive($dir, $vconf);
 		push(@aliases,
 			($redirect->{'code'} && !$redirect->{'alias'} ?
 				$redirect->{'code'}." " : "").
@@ -414,8 +477,18 @@ return "No matching Alias or Redirect found";
 sub modify_redirect
 {
 my ($d, $redirect, $oldredirect) = @_;
-&delete_redirect($d, $oldredirect);
-return &create_redirect($d, $redirect);
+# Replace in a safe order, so if creating the new redirect fails, restore old
+# one so edits are non-destructive
+my $err = &delete_redirect($d, $oldredirect);
+return $err if ($err);
+$err = &create_redirect($d, $redirect);
+if ($err) {
+	my $rerr = &create_redirect($d, $oldredirect);
+	if ($rerr) {
+		$err .= " (failed to restore previous redirect: $rerr)";
+		}
+	}
+return $err;
 }
 
 # get_redirect_root(&domain)
@@ -443,7 +516,7 @@ sub add_wellknown_redirect
 my ($redir) = @_;
 if ($redir->{'path'} eq '/' && !$redir->{'alias'} &&
     !$redir->{'regexp'} && !$redir->{'exact'}) {
-	$redir->{'path'} = '^/(?!.well-known)';
+	$redir->{'path'} = '^/(?!\.well-known)';
 	$redir->{'regexp'} = 1;
 	}
 return $redir;
@@ -455,7 +528,8 @@ sub remove_wellknown_redirect
 {
 my ($redir) = @_;
 if (($redir->{'path'} eq '^/(?!.well-known)' ||
-     $redir->{'path'} eq '^(?!/.well-known)') &&
+     $redir->{'path'} eq '^(?!/.well-known)' ||
+     $redir->{'path'} eq '^/(?!\.well-known)') &&
     !$redir->{'alias'} && $redir->{'regexp'}) {
 	$redir->{'path'} = '/';
 	$redir->{'regexp'} = 0;
@@ -494,8 +568,17 @@ sub is_webmail_redirect
 {
 my ($d, $r) = @_;
 return 0 if (!$r->{'host'});
-return 1 if ($r->{'host'} =~ /^admin\.\Q$d->{'dom'}\E$/);
-return 2 if ($r->{'host'} =~ /^webmail\.\Q$d->{'dom'}\E$/);
+return 0 if ($r->{'path'} ne '/');
+my ($dhost, $dport) =
+	&parse_http_url($r->{'dest'} || '', undef, undef, undef, undef, undef,
+			undef, 1);
+return 0 if (!$dhost || !$dport);
+$dhost = lc($dhost);
+my $basedom = lc($d->{'dom'});
+return 0 if ($dhost ne $basedom && $dhost !~ /\.\Q$basedom\E$/);
+my $rhost = lc($r->{'host'});
+return 1 if ($rhost eq "admin.".$basedom);
+return 2 if ($rhost eq "webmail.".$basedom);
 return 0;
 }
 
