@@ -163,14 +163,13 @@ $default_content_dir = "$module_root_directory/default";
 		      'migrate');
 
 @reseller_modules = ("webminlog", "mailboxes", "bind8", "logviewer", "filemin",
-		     "xterm");
+		     "xterm", "phpini");
 
 $reseller_group_name = "resellers";
 
 @all_template_files = ( "domain-template", "subdomain-template",
 			"user-template", "local-template", "bw-template",
-			"warnbw-template", "framefwd-template",
-			"update-template",
+			"warnbw-template", "update-template",
 			$virtualmin_pro ? ( "reseller-template" ) : ( ) );
 
 $initial_users_dir = "$module_config_directory/initial";
@@ -216,12 +215,13 @@ $plans_dir = "$module_config_directory/plans";
 $extra_users_dir = "$module_config_directory/users";
 
 $extra_admins_dir = "$module_config_directory/admins";
-@all_possible_php_versions = ( 5, 5.2, 5.3, 5.4, 5.5, 5.6,
-                              "7.0", 7.1, 7.2, 7.3, 7.4,
-                              "8.0", 8.1, 8.2, 8.3, 8.4 );
+@all_possible_php_versions = (
+	5, 5.2, 5.3, 5.4, 5.5, 5.6,
+	"7.0", 7.1, 7.2, 7.3, 7.4,
+	"8.0", 8.1, 8.2, 8.3, 8.4, 8.5, 8.6, 8.7, 8.8, 8.9 );
 @all_possible_short_php_versions =
 	&unique(map { int($_) } @all_possible_php_versions);
-$max_php_fcgid_children = $config{'max_php_fcgi_children'} || int(&get_php_max_childred_allowed() * 5) || 5;
+$max_php_fcgid_children = $config{'max_php_fcgi_children'} || int(&get_php_max_children_allowed() * 5) || 5;
 $max_php_fcgid_timeout = $config{'max_php_fcgi_timeout'} || 9999;
 $s3_upload_tries = $config{'upload_tries'} || 3;
 $rs_upload_tries = $config{'upload_tries'} || 3;
@@ -242,7 +242,10 @@ $s3_accounts_dir = "$module_config_directory/s3accounts";
 
 $denied_ssh_group = "deniedssh";
 
-$virtualmin_link = "https://www.virtualmin.com";
+$virtualmin_host_domain = "www.virtualmin.com";
+$virtualmin_link = "https://$virtualmin_host_domain";
+$virtualmin_docs_link = "$virtualmin_link/docs/";
+$virtualmin_account_link = "$virtualmin_link/account/";
 $virtualmin_account_subscriptions = "$virtualmin_link/account/subscriptions/";
 $virtualmin_shop_link = "$virtualmin_link/shop/";
 $virtualmin_shop_link_cat = "$virtualmin_link/product-category/virtualmin/";
@@ -267,33 +270,41 @@ $script_latest_file = "scripts.txt";
 $script_latest_key = "latest-scripts\@virtualmin.com";
 
 $virtualmin_license_file = "/etc/virtualmin-license";
-$upgrade_virtualmin_host_unstable = "software.virtualmin.dev";
-$upgrade_virtualmin_host_rc = "rc.software.virtualmin.dev";
 $upgrade_virtualmin_host = "software.virtualmin.com";
+$upgrade_virtualmin_host_new = "download.virtualmin.com";
+$upgrade_virtualmin_host_rc = "rc.software.virtualmin.dev";
+$upgrade_virtualmin_host_rc_new = "rc.download.virtualmin.dev";
+$upgrade_virtualmin_host_unstable = "software.virtualmin.dev";
+$upgrade_virtualmin_host_unstable_new = "download.virtualmin.dev";
 $upgrade_virtualmin_host_all = join("|", map { quotemeta($_) } (
-	$upgrade_virtualmin_host_unstable,
-	$upgrade_virtualmin_host_rc,
 	$upgrade_virtualmin_host,
+	$upgrade_virtualmin_host_new,
+	$upgrade_virtualmin_host_rc,
+	$upgrade_virtualmin_host_rc_new,
+	$upgrade_virtualmin_host_unstable,
+	$upgrade_virtualmin_host_unstable_new,
 	));
-$virtualmin_apt_auth_dir = "/etc/apt/auth.conf.d";
+
 ($virtualmin_apt_auth_file) =
 	grep { -r $_ } (
-		"$virtualmin_apt_auth_dir/virtualmin-unstable.conf",
-		"$virtualmin_apt_auth_dir/virtualmin-prerelease.conf",
-		"$virtualmin_apt_auth_dir/virtualmin.conf",
+		"/etc/apt/auth.conf.d/virtualmin-unstable.conf",
+		"/etc/apt/auth.conf.d/virtualmin-prerelease.conf",
+		"/etc/apt/auth.conf.d/virtualmin-stable.conf",
+		"/etc/apt/auth.conf.d/virtualmin.conf",
 	);
 ($virtualmin_yum_repo) =
 	grep { -r $_ } (
 		"/etc/yum.repos.d/virtualmin-unstable.repo",
 		"/etc/yum.repos.d/virtualmin-prerelease.repo",
+		"/etc/yum.repos.d/virtualmin-stable.repo",
 		"/etc/yum.repos.d/virtualmin.repo",
 	);
 ($virtualmin_apt_repo) =
 	grep { -r $_ } (
 		"/etc/apt/sources.list.d/virtualmin-unstable.list",
 		"/etc/apt/sources.list.d/virtualmin-prerelease.list",
+		"/etc/apt/sources.list.d/virtualmin-stable.list",
 		"/etc/apt/sources.list.d/virtualmin.list",
-		"/etc/apt/sources.list",
 	);
 
 $upgrade_virtualmin_port = 443;
@@ -490,4 +501,3 @@ return $htext;
 }
 
 1;
-
